//+------------------------------------------------------------------+
//|   Qualitative Quantitative Estimation Indicator for Metatrader 4 |
//|                                     Copyright  2006Roman Ignatov |
//|                                   mailto:roman.ignatov@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  200 Roman Ignatov"
#property link      "mailto:roman.ignatov@gmail.com"

#property indicator_separate_window

#property indicator_buffers 3

#property indicator_color1 PowderBlue
#property indicator_style1 STYLE_SOLID
#property indicator_width1 2

#property indicator_color2 Gold
#property indicator_style2 STYLE_DOT
#property indicator_width2 1

#property indicator_color3 Gold
#property indicator_style3 STYLE_SOLID
#property indicator_width3 1

#property indicator_level1 50.00
#property indicator_levelcolor PowderBlue
#property indicator_levelstyle STYLE_SOLID
#property indicator_levelwidth 1



extern int SF = 5;

extern int RSI_Period = 14;

extern int RSI_Price = PRICE_CLOSE;

extern int MA_Mode = MODE_EMA;

extern double Slow = 2.618;

extern double Fast = 1.236;


// alert

extern bool enableAlerts=true;

extern bool onlySound=true;

extern string soundFile="alert.wav";

int Wilders_Period;
int StartBar;

double TrLevelSlow[];
double TrLevelFast[];
double AtrRsi[];
double MaAtrRsi[];
double Rsi[];
double RsiMa[];

int init()
{
    Wilders_Period = RSI_Period * 2 - 1;
    if (Wilders_Period < SF)
        StartBar = SF;
    else
        StartBar = Wilders_Period;
        
    IndicatorBuffers(6);
    SetIndexBuffer(0, RsiMa);
    SetIndexStyle(0, DRAW_LINE);
    SetIndexLabel(0, "Value 1");
    SetIndexDrawBegin(0, StartBar);
    SetIndexStyle(1, DRAW_LINE);
    SetIndexBuffer(1, TrLevelFast);
    SetIndexLabel(1, "Value 2");
    SetIndexDrawBegin(1, StartBar);
    SetIndexStyle(2, DRAW_LINE);
    SetIndexBuffer(2, TrLevelSlow);
    SetIndexLabel(2, "Value 3");
    SetIndexDrawBegin(2, StartBar);
    SetIndexBuffer(3, AtrRsi);
    SetIndexBuffer(4, MaAtrRsi);
    SetIndexBuffer(5, Rsi);
    IndicatorShortName(StringConcatenate("QQE(", SF, ")"));
    return(0);
}


int start()
{
    int counted, i;
    double rsi0, rsi1, dar, tr, dv;
    double rsi2, dar2, tr2, dv2;
    
    if(Bars <= StartBar)
        return (0);

    counted = IndicatorCounted();
    if(counted < 1)
        for(i = Bars - StartBar; i < Bars; i++)
        {
            TrLevelSlow[i] = 0.0;
            TrLevelFast[i] = 0.0;
            AtrRsi[i] = 0.0;
            MaAtrRsi[i] = 0.0;
            Rsi[i] = 0.0;
            RsiMa[i] = 0.0;
        }
    
    counted = Bars - counted - 1;
        
    for (i = counted; i >= 0; i--)
        Rsi[i] = iRSI(NULL, 0, RSI_Period, RSI_Price, i);

    for (i = counted; i >= 0; i--)
    {
        RsiMa[i] = iMAOnArray(Rsi, 0, SF, 0, MA_Mode, i);
        AtrRsi[i] = MathAbs(RsiMa[i + 1] - RsiMa[i]);
    }

    for (i = counted; i >= 0; i--)
        MaAtrRsi[i] = iMAOnArray(AtrRsi, 0, Wilders_Period, 0, MA_Mode, i);

    i = counted + 1;
    tr = TrLevelSlow[i];
    tr2 = TrLevelFast[i];
    rsi1 = iMAOnArray(Rsi, 0, SF, 0, MA_Mode, i);
    rsi2 = iMAOnArray(Rsi, 0, SF, 0, MA_Mode, i);
    while (i > 0)
    {
        i--;
        rsi0 = iMAOnArray(Rsi, 0, SF, 0, MA_Mode, i);
        dar = iMAOnArray(MaAtrRsi, 0, Wilders_Period, 0, MA_Mode, i) * Slow;

        dv = tr;
        if (rsi0 < tr)
        {
            tr = rsi0 + dar;
            if (rsi1 < dv)
                if (tr > dv)
                    tr = dv;
        }
        else if (rsi0 > tr)
        {
            tr = rsi0 - dar;
            if (rsi1 > dv)
                if (tr < dv)
                    tr = dv;
        }
        TrLevelSlow[i] = tr;
        rsi1 = rsi0;

        dar2 = iMAOnArray(MaAtrRsi, 0, Wilders_Period, 0, MA_Mode, i) * Fast; 
        dv2 = tr2;
        if (rsi0 < tr2)
        {
            tr2 = rsi0 + dar2;
            if (rsi2 < dv2)
                if (tr2 > dv2)
                    tr2 = dv2;
        }
        else if (rsi0 > tr2)
        {
            tr2 = rsi0 - dar2;
            if (rsi2 > dv2)
                if (tr2 < dv2)
                    tr2 = dv2;
        }
        TrLevelFast[i] = tr2;
        rsi2 = rsi0;

        

    }

// tu wstawka z alertem

    if(enableAlerts && isNewBar()) {
    
     if(TrLevelFast[1]>RsiMa[1] && TrLevelFast[2]<RsiMa[2]) {
      if(onlySound) {
       PlaySound(soundFile);
      } else {
       Alert("Cross up on " + Symbol());
      }
     }
     
     if(TrLevelFast[1]<RsiMa[1] && TrLevelFast[2]>RsiMa[2]) {
      if(onlySound) {
       PlaySound(soundFile);
      } else {
       Alert("Cross down on " + Symbol());
      }
     }
         
    }

// koniec wstawki    

    return(0);
}

// wstawka zwiazana z alertem

bool isNewBar() {
 static int prevTime; 
 bool newBar=false; 
 
 if(Time[0]!=prevTime) { 
  newBar=true; 
  prevTime=Time[0]; 
 }
 
 return(newBar);
}

// koniec wstawki